"""
The module contains constants for spell types.
**Spell types**
:param GLOBAL_SPELL: A spell to perform *OUTSIDE* the created adventure. (For example - creating an adventure)
:param LOCAL_SPELL: A spell to perform *INSIDE* the created adventure. (For example - go traveling)
"""
GLOBAL_SPELL = 'GLOBAL_SPELL'
LOCAL_SPELL = 'LOCAL_SPELL'
