"""
The module contains the rules of scraping.
"""
from ptm import Habbit


SALLIES = ()


class Frodo(Habbit):
    brotherhoods = ()
    start_page_url = ''
    path = {}

    def pcik_up(self, result_set):
        pass
