from .start_adventure import StartAdventure
from .create_land import CreateLand
from .create_travel import CreateTravel
from .run_adventure import RunAdventure

spells = (
    StartAdventure,
    CreateLand,
    CreateTravel,
    RunAdventure
)
